#!/bin/bash

####################################################################
#                                                                  #
#         run the script with "./generate.sh inputfile"            #
#                                                                  #
####################################################################

# indel-Seq-Sen can be downloaded from http://bioinfolab.unl.edu/~cstrope/iSG/ 
ISG_HOME="./iSG"
ISG="indel-seq-gen"
OUT_DIR="./raw_data"
branchScale=0.5
# the min/max length of the amino acid sequences 
minLength=100
maxLength=1000
# the min/max gap length is defined in percentage of sequence length
minGapLength=1
maxGapLength=10

arch=$(uname -i)
if [[ $arch == *64* ]]; then
	ISG="indel-seq-gen_64"
else
	ISG="indel-seq-gen_32"
fi

if [ ! -d "$OUT_DIR" ]; then
    mkdir $OUT_DIR
fi
rm stdout.log 2> /dev/null
rm stderr.log 2> /dev/null

echo "generating sequences with $ISG"
counter=0
maxcounter=$(cat $1 | grep "^(" | wc -l)
for tree in $(cat $1 | grep "^(")
do
	tree=$(echo -n $tree | sed 's/\(.*)\)[^)]*;/\1;/g')
	let "counter += 1"
	echo -ne "\r$counter/$maxcounter"
	let "length = ($RANDOM % ($maxLength-$minLength)) + $minLength"
	let "gaplength = ($RANDOM % (($length*$maxGapLength/100)-($length*$minGapLength/100))) + ($length*$minGapLength/100)"
	echo "[$length]{$gaplength,0}" > "temptree"
	echo "$tree" >> "temptree"
	$ISG_HOME/$ISG --matrix PAM --outfile "$counter" --outfile_format FASTA --branch_scale "$branchScale" --select_outputs 010000 --quiet < temptree >> stdout.log 2>> stderr.log
done
echo
rm *.scale_tree 2> /dev/null
rm temptree 2> /dev/null

echo "generating .faa files"
rm $OUT_DIR/*.faa 2> /dev/null
for seqfile in *.seq
do
	seqid=A
	for line in $(cat $seqfile)
	do
		if [[ $line == \>* ]]; then
			seqid=$(echo $line | sed 's/>\([^_]*\)_.*/\1/g')
		fi
		echo "$line" >> "$OUT_DIR/$seqid".faa
	done
done
rm *.seq 2> /dev/null

echo "generating .gff files"
rm $OUT_DIR/*.gff 2> /dev/null
while read line
do
	if [[ $line == Order* ]]; then
		counter=0
		species=$(echo -n $line | sed 's/^Order\s*\([^:]*\):.*/\1/g')
		genes=$(echo -n $line | sed 's/^Order\s*[^:]*:\s*\(.*\)/\1/g' | tr '\r' ' ')
		for gene in $genes
		do
			if [[ $gene == -* ]]; then
				sign="-"
			else
				sign="+"
			fi
			gene=$(echo -n $gene | sed 's/^-*\(.*\)/\1/g')
			let "counter += 1"
			echo -e "gi|12345678|ref|NC_012345.1|\tsim\tCDS\t$counter\t$counter\t.\t$sign\t.\tID=$species"_"$gene;" >> "$OUT_DIR/$species".gff
		done
	fi
done < $1 

